@@ -7,6 +7,7 @@ from fidesctl.cli.cli import (
     delete,
     evaluate,
     generate_dataset,
+    scan,
     annotate_dataset,
     get,
     init_db,
@@ -26,6 +27,7 @@ API_COMMANDS = [
     apply,
     delete,
     generate_dataset,
+    scan,
     get,
     init_db,
     ls,
@@ -47,7 +49,7 @@ API_COMMANDS = [
     "-f",
     "config_path",
     default="",
-    help="Path to a configuration file. Use 'fidesctl view-config' to print the config.",
+    help="Path to a configuration file. Use 'fidesctl view-config' to print the config. Not compatible with the 'fidesctl webserver' subcommand.",
 )
 @click.option(
     "--local",
