@@ -140,6 +140,39 @@ def generate_dataset(
     _generate_dataset.generate_dataset(connection_string, output_filename)
 
 
+@click.command()
+@click.pass_context
+@click.argument("source_type", type=click.Choice(["database"]))
+@click.argument("connection_string", type=str)
+@click.option("-m", "--manifest-dir", type=str, default="")
+@click.option("-c", "--coverage-threshold", type=click.IntRange(0, 100), default=100)
+def scan(
+    ctx: click.Context,
+    source_type: str,
+    connection_string: str,
+    manifest_dir: str,
+    coverage_threshold: int,
+) -> None:
+    """
+    Connect to a database directly via a SQLAlchemy-stlye connection string and
+    compare the database objects to existing datasets.
+
+    If there are fields within the database that aren't listed and categorized
+    within one of the datasets, this counts as lacking coverage.
+
+    Outputs missing fields and has a non-zero exit if coverage is
+    under the stated threshold.
+    """
+    config = ctx.obj["CONFIG"]
+    _generate_dataset.database_coverage(
+        connection_string=connection_string,
+        manifest_dir=manifest_dir,
+        coverage_threshold=coverage_threshold,
+        url=config.cli.server_url,
+        headers=config.user.request_headers,
+    )
+
+
 @click.command()
 @click.pass_context
 @click.argument("input_filename", type=str)
