@@ -1,34 +1,34 @@
 """Module for evaluating policies."""
-from typing import Dict, List, Optional, Callable, cast
-
 import uuid
-import time
+from typing import Callable, Dict, List, Optional, Set, cast
+
 from pydantic import AnyHttpUrl
 
 from fidesctl.cli.utils import handle_cli_response, pretty_echo
 from fidesctl.core import api
-from fidesctl.core.api_helpers import get_server_resources, get_server_resource
+from fidesctl.core.api_helpers import get_server_resource, get_server_resources
 from fidesctl.core.parse import parse
 from fidesctl.core.utils import echo_green, echo_red
+from fideslang.default_taxonomy import DEFAULT_TAXONOMY
 from fideslang.models import (
-    ActionEnum,
-    Evaluation,
     Dataset,
-    StatusEnum,
-    InclusionEnum,
+    Evaluation,
+    MatchesEnum,
     Policy,
     PolicyRule,
     PrivacyDeclaration,
+    StatusEnum,
     System,
     Taxonomy,
+    Violation,
+    ViolationAttributes,
 )
-from fideslang.default_taxonomy import DEFAULT_TAXONOMY
-from fideslang.validation import FidesKey
 from fideslang.relationships import (
     get_referenced_missing_keys,
     hydrate_missing_resources,
 )
 from fideslang.utils import get_resource_by_fides_key
+from fideslang.validation import FidesKey
 
 
 def get_evaluation_policies(
@@ -107,23 +107,6 @@ def validate_policies_exist(policies: List[Policy], evaluate_fides_key: str) ->
         raise SystemExit(1)
 
 
-def validate_supported_policy_rules(policies: List[Policy]) -> None:
-    """
-    Validates that only supported actions are used in taxonomy. Currently
-    evaluations only support REJECT policy actions.
-    see: https://github.com/ethyca/fides/issues/150
-    """
-    for policy in policies:
-        for rule in policy.rules:
-            if rule.action != ActionEnum.REJECT:
-                echo_red(
-                    "Policy ({}) uses unsupported policy action ({}). Only REJECT is currently supported".format(
-                        policy.name, rule.action
-                    )
-                )
-                raise SystemExit(1)
-
-
 def get_fides_key_parent_hierarchy(
     taxonomy: Taxonomy, fides_key: str
 ) -> List[FidesKey]:
@@ -157,77 +140,39 @@ def get_fides_key_parent_hierarchy(
 def compare_rule_to_declaration(
     rule_types: List[FidesKey],
     declaration_type_hierarchies: List[List[FidesKey]],
-    rule_inclusion: InclusionEnum,
-) -> bool:
+    rule_match: MatchesEnum,
+) -> Set[str]:
     """
     Compare the list of fides_keys within the rule against the list
-    of fides_keys hierarchies from the declaration and uses the rule's inclusion
-    field to determine whether the rule is triggered or not.
+    of fides_keys hierarchies from the declaration and uses the rule's matches
+    field to determine whether the rule is triggered or not. Returns the offending
+    keys, prioritizing the first descendant in the hierarchy.
     """
-    inclusion_map: Dict[InclusionEnum, Callable] = {
-        InclusionEnum.ANY: any,
-        InclusionEnum.ALL: all,
-        InclusionEnum.NONE: lambda x: not any(x),
+    matched_declaration_types = set()
+    mismatched_declaration_types = set()
+    for declaration_type_hierarchy in declaration_type_hierarchies:
+        declared_declaration_type = declaration_type_hierarchy[0]
+        if len(set(declaration_type_hierarchy).intersection(set(rule_types))) > 0:
+            matched_declaration_types.add(declared_declaration_type)
+        else:
+            mismatched_declaration_types.add(declared_declaration_type)
+
+    matches_map: Dict[MatchesEnum, Callable] = {
+        # any matches return matching declared values as violations
+        MatchesEnum.ANY: lambda: matched_declaration_types,
+        # all matches return matching declared values as violations if all values match rule values
+        MatchesEnum.ALL: lambda: matched_declaration_types
+        if len(matched_declaration_types) == len(declaration_type_hierarchies)
+        else set(),
+        # none matches return mismatched declared values as violations if none of the values matched rule values
+        MatchesEnum.NONE: lambda: mismatched_declaration_types
+        if not any(matched_declaration_types)
+        else set(),
+        # other matches return mismatched declared values as violations
+        MatchesEnum.OTHER: lambda: mismatched_declaration_types,
     }
 
-    matching_data_categories = [
-        bool(len(set(data_category_hierarchy).intersection(set(rule_types))) > 0)
-        for data_category_hierarchy in declaration_type_hierarchies
-    ]
-    result = inclusion_map[rule_inclusion](matching_data_categories)
-    return result
-
-
-def validate_fides_keys_exist_for_evaluation(
-    taxonomy: Taxonomy,
-    policy_rule: PolicyRule,
-    data_subjects: List[str],
-    data_categories: List[str],
-    data_qualifier: str,
-    data_use: str,
-) -> None:
-    """
-    Validates that keys used in evaluations are valid in the taxonomy
-    """
-    missing_key_errors = []
-
-    categories_keys = [category.fides_key for category in taxonomy.data_category]
-    for missing_data_category in set(
-        data_categories + policy_rule.data_categories.values
-    ) - set(categories_keys):
-        missing_key_errors.append(
-            "Missing DataCategory ({})".format(missing_data_category)
-        )
-
-    subject_keys = [subject.fides_key for subject in taxonomy.data_subject]
-    for missing_data_subject in set(
-        data_subjects + policy_rule.data_subjects.values
-    ) - set(subject_keys):
-        missing_key_errors.append(
-            "Missing DataSubject ({})".format(missing_data_subject)
-        )
-
-    qualifiers_keys = [qualifier.fides_key for qualifier in taxonomy.data_qualifier]
-    for missing_data_qualifier in set(
-        [data_qualifier, policy_rule.data_qualifier]
-    ) - set(qualifiers_keys):
-        missing_key_errors.append(
-            "Missing DataQualifier ({})".format(missing_data_qualifier)
-        )
-
-    data_use_keys = [data_use.fides_key for data_use in taxonomy.data_use]
-    for missing_data_use in set([data_use] + policy_rule.data_uses.values) - set(
-        data_use_keys
-    ):
-        missing_key_errors.append("Missing DataUse ({})".format(missing_data_use))
-
-    if missing_key_errors:
-        echo_red(
-            "Found missing keys referenced in taxonomy \n{}".format(
-                "\n ".join(missing_key_errors)
-            )
-        )
-        raise SystemExit(1)
+    return matches_map[rule_match]()
 
 
 def evaluate_policy_rule(
@@ -237,64 +182,76 @@ def evaluate_policy_rule(
     data_categories: List[str],
     data_qualifier: str,
     data_use: str,
-) -> bool:
+    declaration_violation_message: str,
+) -> List[Violation]:
     """
     Given data subjects, data categories, data qualifier and data use,
     builds hierarchies of applicable types and evaluates the result of a
     policy rule
     """
-    validate_fides_keys_exist_for_evaluation(
-        taxonomy=taxonomy,
-        policy_rule=policy_rule,
-        data_subjects=data_subjects,
-        data_categories=data_categories,
-        data_qualifier=data_qualifier,
-        data_use=data_use,
-    )
-
     category_hierarchies = [
         get_fides_key_parent_hierarchy(
             taxonomy=taxonomy, fides_key=declaration_category
         )
         for declaration_category in data_categories
     ]
-    data_category_result = compare_rule_to_declaration(
+    data_category_violations = compare_rule_to_declaration(
         rule_types=policy_rule.data_categories.values,
         declaration_type_hierarchies=category_hierarchies,
-        rule_inclusion=policy_rule.data_categories.inclusion,
+        rule_match=policy_rule.data_categories.matches,
     )
 
     # A declaration only has one data use, so its hierarchy gets put in a list
     data_use_hierarchies = [
         get_fides_key_parent_hierarchy(taxonomy=taxonomy, fides_key=data_use)
     ]
-    data_use_result = compare_rule_to_declaration(
+    data_use_violations = compare_rule_to_declaration(
         rule_types=policy_rule.data_uses.values,
         declaration_type_hierarchies=data_use_hierarchies,
-        rule_inclusion=policy_rule.data_uses.inclusion,
+        rule_match=policy_rule.data_uses.matches,
     )
 
     # A data subject does not have a hierarchical structure
-    data_subject_result = compare_rule_to_declaration(
+    data_subject_violations = compare_rule_to_declaration(
         rule_types=policy_rule.data_subjects.values,
         declaration_type_hierarchies=[[data_subject] for data_subject in data_subjects],
-        rule_inclusion=policy_rule.data_subjects.inclusion,
+        rule_match=policy_rule.data_subjects.matches,
     )
 
-    data_qualifier_result = (
+    data_qualifier_violation = (
         policy_rule.data_qualifier
         in get_fides_key_parent_hierarchy(taxonomy=taxonomy, fides_key=data_qualifier)
     )
 
     evaluation_result = all(
         [
-            data_category_result,
-            data_subject_result,
-            data_use_result,
-            data_qualifier_result,
+            data_category_violations,
+            data_use_violations,
+            data_subject_violations,
+            data_qualifier_violation,
         ]
     )
-    return evaluation_result
+
+    if evaluation_result:
+        violations = [
+            Violation(
+                detail="{}. Violated usage of data categories ({}) with qualifier ({}) for data uses ({}) and subjects ({})".format(
+                    declaration_violation_message,
+                    ",".join(data_category_violations),
+                    data_qualifier,
+                    ",".join(data_use_violations),
+                    ",".join(data_subject_violations),
+                ),
+                violating_attributes=ViolationAttributes(
+                    data_categories=data_category_violations,
+                    data_uses=data_use_violations,
+                    data_subjects=data_subject_violations,
+                    data_qualifier=data_qualifier,
+                ),
+            )
+        ]
+        return violations
+    return []
 
 
 def get_dataset_by_fides_key(taxonomy: Taxonomy, fides_key: str) -> Optional[Dataset]:
@@ -317,76 +274,81 @@ def evaluate_dataset_reference(
     policy_rule: PolicyRule,
     privacy_declaration: PrivacyDeclaration,
     dataset: Dataset,
-) -> List[str]:
+) -> List[Violation]:
     """
     Evaluates the contraints of a given rule and dataset that was referenced
     from a given privacy declaration
     """
-    evaluation_detail_list = []
+    evaluation_violation_list = []
     if dataset.data_categories:
-        dataset_result = evaluate_policy_rule(
+
+        dataset_violation_message = "Declaration ({}) of system ({}) failed rule ({}) from policy ({}) for dataset ({})".format(
+            privacy_declaration.name,
+            system.fides_key,
+            policy_rule.name,
+            policy.fides_key,
+            dataset.fides_key,
+        )
+
+        dataset_result_violations = evaluate_policy_rule(
             taxonomy=taxonomy,
             policy_rule=policy_rule,
             data_subjects=privacy_declaration.data_subjects,
             data_categories=dataset.data_categories,
             data_qualifier=dataset.data_qualifier,
             data_use=privacy_declaration.data_use,
+            declaration_violation_message=dataset_violation_message,
         )
 
-        if dataset_result:
-            evaluation_detail_list += [
-                "Declaration ({}) of System ({}) failed Rule ({}) from Policy ({}) for Dataset ({})".format(
-                    privacy_declaration.name,
-                    system.fides_key,
-                    policy_rule.name,
-                    policy.fides_key,
-                    dataset.fides_key,
-                )
-            ]
+        evaluation_violation_list += dataset_result_violations
+
     for collection in dataset.collections:
+
+        collection_violation_message = "Declaration ({}) of system ({}) failed rule ({}) from policy ({}) for dataset collection ({})".format(
+            privacy_declaration.name,
+            system.fides_key,
+            policy_rule.name,
+            policy.fides_key,
+            collection.name,
+        )
+
         if collection.data_categories:
-            dataset_collection_result = evaluate_policy_rule(
+            dataset_collection_result_violations = evaluate_policy_rule(
                 taxonomy=taxonomy,
                 policy_rule=policy_rule,
                 data_subjects=privacy_declaration.data_subjects,
                 data_categories=collection.data_categories,
                 data_qualifier=collection.data_qualifier,
                 data_use=privacy_declaration.data_use,
+                declaration_violation_message=collection_violation_message,
             )
 
-            if dataset_collection_result:
-                evaluation_detail_list += [
-                    "Declaration ({}) of System ({}) failed Rule ({}) from Policy ({}) for DatasetCollection ({})".format(
-                        privacy_declaration.name,
-                        system.fides_key,
-                        policy_rule.name,
-                        policy.fides_key,
-                        collection.name,
-                    )
-                ]
+            evaluation_violation_list += dataset_collection_result_violations
 
         for field in collection.fields:
+
+            field_violation_message = "Declaration ({}) of system ({}) failed rule ({}) from policy ({}) for dataset field ({})".format(
+                privacy_declaration.name,
+                system.fides_key,
+                policy_rule.name,
+                policy.fides_key,
+                field.name,
+            )
+
             if field.data_categories:
-                field_result = evaluate_policy_rule(
+                field_result_violations = evaluate_policy_rule(
                     taxonomy=taxonomy,
                     policy_rule=policy_rule,
                     data_subjects=privacy_declaration.data_subjects,
                     data_categories=field.data_categories,
                     data_qualifier=field.data_qualifier,
                     data_use=privacy_declaration.data_use,
+                    declaration_violation_message=field_violation_message,
                 )
 
-                if field_result:
-                    evaluation_detail_list += [
-                        "Declaration ({}) of System ({}) failed Rule ({}) from Policy ({}) for DatasetField ({})".format(
-                            privacy_declaration.name,
-                            system.fides_key,
-                            policy_rule.name,
-                            policy.fides_key,
-                            field.name,
-                        )
-                    ]
-    return evaluation_detail_list
+                evaluation_violation_list += field_result_violations
+
+    return evaluation_violation_list
 
 
 def evaluate_privacy_declaration(
@@ -395,37 +357,40 @@ def evaluate_privacy_declaration(
     system: System,
     policy_rule: PolicyRule,
     privacy_declaration: PrivacyDeclaration,
-) -> List[str]:
+) -> List[Violation]:
     """
     Evaluates the contraints of a given rule and privacy declaration. This
     includes additional data set references
     """
-    evaluation_detail_list = []
-    declaration_result = evaluate_policy_rule(
+    evaluation_violation_list = []
+
+    declaration_violation_message = (
+        "Declaration ({}) of system ({}) failed rule ({}) from policy ({})".format(
+            privacy_declaration.name,
+            system.fides_key,
+            policy_rule.name,
+            policy.fides_key,
+        )
+    )
+
+    declaration_result_violations = evaluate_policy_rule(
         taxonomy=taxonomy,
         policy_rule=policy_rule,
         data_subjects=privacy_declaration.data_subjects,
         data_categories=privacy_declaration.data_categories,
         data_qualifier=privacy_declaration.data_qualifier,
         data_use=privacy_declaration.data_use,
+        declaration_violation_message=declaration_violation_message,
     )
 
-    if declaration_result:
-        evaluation_detail_list += [
-            "Declaration ({}) of System ({}) failed Rule ({}) from Policy ({})".format(
-                privacy_declaration.name,
-                system.fides_key,
-                policy_rule.name,
-                policy.fides_key,
-            )
-        ]
+    evaluation_violation_list += declaration_result_violations
 
     for dataset_reference in privacy_declaration.dataset_references or []:
         dataset = get_dataset_by_fides_key(
             taxonomy=taxonomy, fides_key=dataset_reference
         )
         if dataset:
-            evaluation_detail_list += evaluate_dataset_reference(
+            evaluation_violation_list += evaluate_dataset_reference(
                 taxonomy=taxonomy,
                 policy=policy,
                 system=system,
@@ -435,12 +400,12 @@ def evaluate_privacy_declaration(
             )
         else:
             echo_red(
-                "Dataset ({}) referenced in Declaration ({}) could not be found in taxonomy".format(
+                "Dataset ({}) referenced in declaration ({}) could not be found in taxonomy".format(
                     dataset_reference, privacy_declaration.name
                 )
             )
             raise SystemExit(1)
-    return evaluation_detail_list
+    return evaluation_violation_list
 
 
 def execute_evaluation(taxonomy: Taxonomy) -> Evaluation:
@@ -448,13 +413,13 @@ def execute_evaluation(taxonomy: Taxonomy) -> Evaluation:
     Check the stated constraints of each Privacy Policy's rules against
     each system's privacy declarations.
     """
-    evaluation_detail_list = []
+    evaluation_violation_list = []
     for policy in taxonomy.policy:
         for rule in policy.rules:
             for system in taxonomy.system:
                 for declaration in system.privacy_declarations:
 
-                    evaluation_detail_list += evaluate_privacy_declaration(
+                    evaluation_violation_list += evaluate_privacy_declaration(
                         taxonomy=taxonomy,
                         policy=policy,
                         system=system,
@@ -462,15 +427,13 @@ def execute_evaluation(taxonomy: Taxonomy) -> Evaluation:
                         privacy_declaration=declaration,
                     )
     status_enum = (
-        StatusEnum.FAIL if len(evaluation_detail_list) > 0 else StatusEnum.PASS
+        StatusEnum.FAIL if len(evaluation_violation_list) > 0 else StatusEnum.PASS
     )
     new_uuid = str(uuid.uuid4()).replace("-", "_")
-    timestamp = str(time.time()).split(".")[0]
-    generated_key = f"{new_uuid}_{timestamp}"
     evaluation = Evaluation(
-        fides_key=generated_key,
+        fides_key=new_uuid,
         status=status_enum,
-        details=evaluation_detail_list,
+        violations=evaluation_violation_list,
     )
     return evaluation
 
@@ -488,7 +451,12 @@ def populate_referenced_keys(
     Recursively calls itself after every hydration to make sure there are no new missing keys.
     """
     missing_resource_keys = get_referenced_missing_keys(taxonomy)
-    if missing_resource_keys and set(missing_resource_keys) != set(last_keys):
+    keys_not_found = set(last_keys).intersection(set(missing_resource_keys))
+    if keys_not_found:
+        echo_red(f"Missing resource keys: {keys_not_found}")
+        raise SystemExit(1)
+
+    if missing_resource_keys:
         taxonomy = hydrate_missing_resources(
             url=url,
             headers=headers,
@@ -539,10 +507,9 @@ def evaluate(
         )
 
     validate_policies_exist(policies=policies, evaluate_fides_key=policy_fides_key)
-    validate_supported_policy_rules(policies=policies)
     echo_green(
-        "Evaluating the following Policies:\n- {}".format(
-            "\n- ".join([key.fides_key for key in taxonomy.policy])
+        "Evaluating the following policies:\n- {}".format(
+            "\n- ".join([key.fides_key for key in policies])
         )
     )
     print("-" * 10)
