@@ -5,7 +5,6 @@ Creates data visualizations for hierarchical Fides resource types.
 from typing import Generator, List, Dict
 
 import plotly
-import plotly.express as px
 import plotly.graph_objects as go
 
 from fidesctl.core import config
@@ -110,89 +109,24 @@ def hierarchy_figures(
     )
 
 
-def sunburst_plot(
-    categories: List[dict], resource_type: str, json_out: bool = False
-) -> str:
+def create_hierarchical_dict(data: dict, keys: List) -> None:
     """
-    Create a sunburst plot from data categories yaml file
-    Reference: https://plotly.com/python/sunburst-charts/
+    Create a nested dictionary given a list of strings as a key path
     Args:
-        categories: list of the dictionaries for each taxonomy member
-        resource_type: the name of the resource type
-        json_out: Flag to return a json representation of the visualization
+        data: Dictionary to contain the nested dictionary as it's built
+        keys: List of keys that equates to the 'path' down the nested dictionary
 
     Returns:
-        Json representation of the figure if `json_out` is True, html otherwise
+        None
     """
-
-    # add color map
-    for category in categories:
-        category["color"] = category[FIDES_KEY_NAME].split(".")[0]
-
-    fig = px.sunburst(
-        categories, names=FIDES_KEY_NAME, parents=FIDES_PARENT_NAME, color="color"
-    )
-    fig.update_layout(
-        title_text=f'Fides {resource_type.replace("_", " ").title()} Hierarchy',
-        font_size=10,
-    )
-
-    if json_out:
-        return fig.to_json()
-    return fig.to_html()
-
-
-def sankey_plot(
-    categories: List[dict], resource_type: str, json_out: bool = False
-) -> str:
-    """
-    Create a sankey plot from data categories yaml file
-    Reference: https://plotly.com/python/sankey-diagram/
-    Args:
-        categories: list of the dictionaries for each taxonomy member
-        resource_type: the name of the resource type
-        json_out: Flag to return a json representation of the visualization
-
-    Returns:
-        Json representation of the figure if `json_out` is True, html otherwise
-    """
-
-    fides_key_dict = {v[FIDES_KEY_NAME]: i for i, v in enumerate(categories)}
-    source = []
-    target = []
-
-    for category in categories:
-        if FIDES_PARENT_NAME in category.keys():
-            if category[FIDES_PARENT_NAME]:
-                source.append(fides_key_dict[category[FIDES_PARENT_NAME]])
-                target.append(fides_key_dict[category[FIDES_KEY_NAME]])
-
-    fig = go.Figure(
-        data=[
-            go.Sankey(
-                valueformat=".1f",
-                valuesuffix="%",
-                node=dict(
-                    pad=15,
-                    thickness=20,
-                    line=dict(color="black", width=0.5),
-                    label=list(fides_key_dict.keys()),
-                    color="blue",  # Maybe make this 'ethyca blue'?
-                    hovertemplate="%{label}",
-                ),
-                link=dict(source=source, target=target, value=target),
-            )
-        ]
-    )
-
-    fig.update_layout(
-        title_text=f'Fides {resource_type.replace("_", " ").title()} Hierarchy',
-        font_size=10,
-    )
-
-    if json_out:
-        return fig.to_json()
-    return fig.to_html()
+    for key in keys:
+        if key in data:
+            if key == keys[-1]:
+                # we've reached the end of the path (no more children)
+                data[key] = {}
+            data = data[key]
+        else:
+            data[key] = {}
 
 
 def convert_categories_to_nested_dict(categories: List[dict]) -> dict:
@@ -214,30 +148,7 @@ def convert_categories_to_nested_dict(categories: List[dict]) -> dict:
 
     Args:
         categories : list of dictionaries containing each entry from a catalog yaml file
-
-    Returns:
-
     """
-
-    def create_hierarchical_dict(data: dict, keys: List) -> None:
-        """
-        Create a nested dictionary given a list of strings as a key path
-        Args:
-            data: Dictionary to contain the nested dictionary as it's built
-            keys: List of keys that equates to the 'path' down the nested dictionary
-
-        Returns:
-            None
-        """
-        for key in keys:
-            if key in data:
-                if key == keys[-1]:
-                    # we've reached the end of the path (no more children)
-                    data[key] = {}
-                data = data[key]
-            else:
-                data[key] = {}
-
     nested_output: Dict[Dict, Dict] = {}
     for category in categories:
         if FIDES_PARENT_NAME not in category:
@@ -248,6 +159,26 @@ def convert_categories_to_nested_dict(categories: List[dict]) -> dict:
     return nested_output
 
 
+def nest_to_html(nested_dict: dict, indent_factor: int) -> Generator:
+    """
+    Create the html
+    Args:
+        nested_dict: nested dictionary for keys to convert to html list object
+        indent_factor: spacing multiplier
+
+    Returns:
+        HTML string containing a nested, unordered list of the nested dictionary keys
+    """
+    spacing = "   " * indent_factor
+    for key, value in nested_dict.items():
+        yield "{}<li>{}</li>".format(spacing, key)
+        if isinstance(value, dict):
+            yield "{spacing}<ul>\n{member}\n{spacing}</ul>".format(
+                spacing=spacing,
+                member="\n".join(nest_to_html(value, indent_factor + 1)),
+            )
+
+
 def nested_categories_to_html_list(
     categories: List[dict], resource_type: str, indent: int = 1
 ) -> str:
@@ -257,31 +188,8 @@ def nested_categories_to_html_list(
         categories: list of the dictionaries for each taxonomy member
         resource_type: the name of the resource type
         indent: spacing multiplier
-
-    Returns:
-
     """
     nested_categories = convert_categories_to_nested_dict(categories)
-
-    def nest_to_html(nested_dict: dict, indent_factor: int) -> Generator:
-        """
-        Create the html
-        Args:
-            nested_dict: nested dictionary for keys to convert to html list object
-            indent_factor: spacing multiplier
-
-        Returns:
-            HTML string containing a nested, unordered list of the nested dictionary keys
-        """
-        spacing = "   " * indent_factor
-        for key, value in nested_dict.items():
-            yield "{}<li>{}</li>".format(spacing, key)
-            if isinstance(value, dict):
-                yield "{spacing}<ul>\n{member}\n{spacing}</ul>".format(
-                    spacing=spacing,
-                    member="\n".join(nest_to_html(value, indent_factor + 1)),
-                )
-
     header = f'<h2>Fides {resource_type.replace("_", " ").title()} Hierarchy</h2>'
     categories_tree = "\n".join(nest_to_html(nested_categories, indent))
     return f"{header}\n{categories_tree}"
